import { DeepRequired } from 'ts-essentials';
import { Algorithm } from 'jsonwebtoken';
import { CompressionOptions, TLSOptions } from "ziron-worker";
export interface ServerConfig {
    /**
     * @description
     * An id that is associated with this specific server instance.
     * The id should be unique for every instance in a cluster.
     * If you don't provide an instance id,
     * a random id will be generated using the package "uniqid".
     * @default random uniqid
     */
    id?: string;
    /**
     * @description
     * The port where the server should listen.
     * @default 3000
     */
    port?: number;
    /**
     * @description
     * The hostname of the server.
     * @default 'localhost'
     */
    hostname?: string;
    /**
     * @description
     * The URL path of the server.
     * Notice multiple slashes at the end are not supported and will be removed.
     * @default ''
     */
    path?: string;
    /**
     * @description
     * This property can be used to specify a license.
     * A valid license is a requirement to use this instance on a public server or for commercial usage.
     * For more information, look here:
     * https://github.com/ZationServer/zation/wiki/Zation-End-User-License-Agreement
     * @default Free Trial
     */
    license?: string;
    /**
     * @description
     * With this property, you can specify what origins are allowed to communicate to the server.
     * You can specify the port and hostname.
     * Also, a star can be used as a wild card for any port or any hostname.
     * @example
     * //allow all origins
     * origins: null, or
     * origins: '*:*',
     *
     * //allow all with hostname example.de
     * origins: 'example.de:*', or
     * origins: 'example.de'
     *
     * //allow all with port 80
     * origins: '*:80'
     *
     * //allow only hostname example.de on port 80
     * origins: 'example.de:80'
     *
     * //allow all with hostname example.de or example2.de
     * origins: ['example.de:*','example2.de']
     *
     * @default null (all allowed)
     */
    origins?: string | null | string[];
    /**
     * @description
     * Tags can be used to add meta information to the server instance.
     * A tag is a key or a key-value pair.
     * The defined tags can be displayed in the panel.
     *
     * Tags can be provided as an array of tuples.
     * It is also possible to define them with a string, which makes it perfect for using an environment variable.
     * The string should contain the tags comma-separated, and a key and value can be separated by a ":".
     */
    tags?: ([string, string?][]) | string;
    /**
     * @description
     * Indicates if the debug mode is active.
     * In debug mode the server will console log information about the current status and actions of connected clients.
     * The debug mode is automatically deactivated when the server runs in production mode.
     * @default false
     */
    debug?: boolean;
    /**
     * @description
     * Indicates if the launch debug mode is active.
     * In launch debug mode, the server will console log all steps for
     * launching the server and how much time each step took.
     * The launch debug mode is automatically deactivated when the server runs in production mode.
     * @default false
     */
    launchDebug?: boolean;
    /**
     * @description
     * Indicates if the server should console log
     * config warnings after checking the configurations.
     * @default true
     */
    showConfigWarnings?: boolean;
    /**
     * @description
     * This option lets you skip the config check on the server start.
     * It's not recommended skipping it, but if the process takes a lot of time, you can do it.
     * Notice that you still should check the configurations with the npm check command after every change.
     * @default true
     */
    checkConfigs?: boolean;
    /**
     * @description
     * The timezone of the server,
     * it affects the calculation of the background task time.
     * @default Zation will guess the timezone if it fails the default is 'Europe/Berlin'.
     */
    timeZone?: string;
    /**
     * @description
     * Defines TLS options.
     * Provide these options to enable TLS 1.3 messaging encryption.
     * @default null
     */
    tls?: TLSOptions | null;
    /**
     * @description
     * Indicates if the server should send error descriptions back to the client.
     * @default false
     */
    sendErrorDescription?: boolean;
    /**
     * @description
     * The default API level a client will get when there is no API level provided in the request or connection.
     * Notice that the value must be greater than 0 and needs to be an integer.
     * @default 1
     */
    defaultClientApiLevel?: number;
    /**
     * @description
     * Options for the zation panel that helps to analyse/check the server usage
     * via a web application that is hosted by the server.
     * Works also in cluster mode.
     * @default {active: false}
     */
    panel?: PanelOptions;
    /**
     * @description
     * Specifies the auth options used to sign and verify JSON web tokens.
     * @default {
     *  algorithm: 'HS256'
     *  secretKey: 256 bits cryptographically random hex string
     *  defaultExpiry: 86400,
     *  authTokenExpireCheckInterval: 12000
     * }
     */
    auth?: AuthOptions;
    /**
     * @description
     * Specifies whether the server should use permessage-deflate and
     * when to compress messages, and what compressor should be used.
     * @default {
     *     active: true,
     *     compressor: DedicatedCompressor4KB,
     *     alwaysCompressBatches: false,
     *     minBytes: 104857,
     *     minLength: 20000
     * }
     */
    compression?: CompressionOptions;
    /**
     * @description
     * The limit of how many checks can be made in one validation request.
     * @default 5
     */
    validationCheckLimit?: number;
    /**
     * @description
     * The limit of how many databoxes
     * a socket can connect to simultaneously.
     * @default 30
     */
    socketDataboxLimit?: number;
    /**
     * @description
     * The limit of how many channels
     * a socket can subscribe to simultaneously.
     * @default 30
     */
    socketChannelLimit?: number;
    /**
     * @description
     * This property gives you the possibility to set variables in the
     * server config which you can access with the Bag.
     * @example
     * //define
     * variables: {
     *    myVar: 'someValue'
     * }
     * //access
     * bag.getServerConfigVariables().myVar;
     */
    variables?: object;
    /**
     * @description
     * Specifies the log options.
     */
    log?: LogOptions;
    /**
     * @description
     * Specifies if the complete component
     * API should be logged at server start.
     * @default true
     */
    logComponentApi?: boolean;
    /**
     * @description
     * Defines the default timeout in milliseconds for
     * receiving the response of an invoke.
     * The timeout only starts when the data of the invoke is completely transmitted,
     * and all containing streams are closed.
     * Notice that an individual response timeout can be specified for
     * an invoke that overrides this option value.
     * @default 7000
     */
    responseTimeout?: number;
    /**
     * @description
     * Specifies the interval in that the server pings the client sockets.
     * The client sockets need to send a pong before the next ping
     * is sent otherwise the socket will be disconnected.
     * @default 8000
     */
    pingInterval?: number;
    /**
     * @description
     * Specifies the maximum allowed message size in bytes.
     * The specified value also influences the max
     * batch size of text and binary packages.
     * @default 4194304 (4 MB)
     */
    maxPayloadSize?: number;
    /**
     * @description
     * The maximum send backpressure per socket that is allowed.
     * When the limit is exceeded, the socket connection will be dropped.
     * Notice that streams will be paused when the backpressure is more
     * than 50% of the maximum backpressure.
     * They continue automatically when the backpressure drains.
     * Also, the batching buffer max size depends on the value.
     * When the batching buffer exceeds 70% of the maximum backpressure,
     * the buffer will be automatically flushed.
     * @default 6291456 (6 MB)
     */
    maxBackpressure?: number;
    /**
     * @description
     * Advanced option.
     * Defines the timeout in milliseconds for receiving
     * the referenced binary content packet of a text packet.
     * @default 10000
     */
    binaryContentPacketTimeout?: number;
    /**
     * @description
     * Advanced option.
     * This option defines how many
     * streams are allowed in a package.
     * @default 20
     */
    streamsPerPackageLimit?: number;
    /**
     * @description
     * Advanced option.
     * This option species if chunks
     * of streams can contain streams.
     * @default false
     */
    chunksCanContainStreams?: boolean;
    /**
     * @description
     * Specifies the join token.
     * The token will be used to join a cluster.
     * It contains a secret and the address of the state server.
     * @default null
     */
    join?: string | null;
    /**
     * @description
     * This property indicates if the server should share the auth
     * options for verifying and sign tokens in a cluster.
     * The options of the first server that has joined the cluster
     * will be used for all other servers in the cluster. When the option is enabled,
     * the server will load the shared auth options.
     * That is useful if you are using generated auth keys that will change on every server start.
     * That helps to make the token valid on every server in the cluster,
     * even if the other servers had different auth options before joining the cluster.
     * @default true
     */
    clusterShareTokenAuth?: boolean;
    /**
     * @description
     * Defines the max size of clients to a broker instance.
     * The concrete max size is determined with the count of broker instances.
     * @default 12
     */
    brokerClusterClientMaxPoolSize?: number;
}
export interface AuthOptions {
    /**
     * @description
     * The algorithm that will be used to sign and verify jwt tokens.
     * @default 'HS256'
     */
    algorithm?: Algorithm;
    /**
     * @description
     * The secret key that is used to encrypt/decrypt auth tokens.
     * The key can be automatically shared between more server instances.
     * If you want to use RSA or ECDSA, you should provide a authPrivateKey and authPublicKey instead of a authKey.
     * @default 256 bits cryptographically random hex string.
     */
    secretKey?: string;
    /**
     * @description
     * The default expiry of tokens in seconds.
     * @default 86400
     */
    defaultExpiry?: number;
    /**
     * @description
     * The interval when the server checks for expired auth tokens.
     * @default 12000ms
     */
    expireCheckInterval?: number;
    /**
     * @description
     * The private secret key to signing the jwt tokens.
     * For using asymmetric encryption, you also need to define the
     * public key and change the algorithm to a proper one, e.g. RSA or ECDSA.
     * @default null
     */
    privateKey?: string | null;
    /**
     * @description
     * The public secret key to verify the jwt tokens.
     * For using asymmetric encryption, you also need to define the
     * private key and change the algorithm to a proper one, e.g. RSA or ECDSA.
     * @default null
     */
    publicKey?: string | null;
    /**
     * @description
     * Defines the version of the auth token.
     * The server will only allow auth tokens with the same version.
     * It can be helpful to increase the version when you have changed
     * the token structure to make the old tokens invalid.
     * @default 1
     */
    tokenVersion?: number;
}
export interface LogOptions {
    /**
     * @description
     * This property specifies options for
     * the logging from the server into the console (local).
     * @default Active with the highest log level.
     */
    console?: ConsoleLogOptions;
    /**
     * @description
     * This property specifies options for
     * the logging from the server into the log file (local).
     * @default Deactivated
     */
    file?: FileLogOptions;
    /**
     * @description
     * This property specifies options for
     * the logging from the server into the panel (remote).
     * @default Active with the log level 5.
     */
    panel?: PanelLogOptions;
    /**
     * @description
     * Specifies if global logging should be used when it is not explicitly specified.
     * Global logging combines local (console or file) and remote (panel) logging.
     * @default false
     */
    defaultGlobal?: boolean;
}
export interface ConsoleLogOptions {
    /**
     * @description
     * This property indicates if the server
     * should log information into the console.
     * @default true
     */
    active?: boolean;
    /**
     * @description
     * Defines the log level for log information into the console.
     * 6: Log Category: DebugInfo (only in debug mode), LaunchDebugInfo (only in launch debug mode)
     * 5: Log Category: Info, Busy
     * 4: Log Category: Warn
     * 3: Log Category: Errors
     * 2: Log Category: Failed, Fatal
     * 1: Log Category: Active
     * 0: Log: Nothing
     * @default 6
     */
    logLevel?: number;
}
export interface FileLogOptions {
    /**
     * @description
     * Defines if the server should log information into a log file.
     * @default false
     */
    active?: boolean;
    /**
     * @description
     * Defines the log level for log information into the log file.
     * 6: Log Category: DebugInfo (only in debug mode), LaunchDebugInfo (only in launch debug mode)
     * 5: Log Category: Info, Busy
     * 4: Log Category: Warn
     * 3: Log Category: Errors
     * 2: Log Category: Failed, Fatal
     * 1: Log Category: Active
     * 0: Log: Nothing
     * @default 5
     */
    logLevel?: number;
    /**
     * @description
     * Specifies the log file path but notice that the path is relative to the running directory.
     * @default ''
     */
    filePath?: string;
    /**
     * @description
     * Defines options for download the log file via HTTP.
     * @default active with no access key
     */
    download?: {
        /**
         * @description
         * Defines if the log file should be downloadable via HTTP.
         * You can use the property accessKey to set a secret password for access the download.
         * @default true
         */
        active?: boolean;
        /**
         * @description
         * Defines a key to access the log file download.
         * The secret key will appear in the URL for access the log file via HTTP.
         * @default ''
         * @example
         * Log: http://localhost:3001/log/theKey
         */
        accessKey?: string;
    };
}
export interface PanelLogOptions {
    /**
     * @description
     * This property indicates if the server
     * should log information into the panel.
     * Notice that panel logging is not local logging.
     * So you either need to specify global logging explicitly or
     * activate the defaultGlobal mode to log a message into the panel.
     * @default true
     */
    active?: boolean;
    /**
     * @description
     * Defines the log level for log information into the panel.
     * 6: Log Category: DebugInfo (only in debug mode), LaunchDebugInfo (only in launch debug mode)
     * 5: Log Category: Info, Busy
     * 4: Log Category: Warn
     * 3: Log Category: Errors
     * 2: Log Category: Failed, Fatal
     * 1: Log Category: Active
     * 0: Log: Nothing
     * @default 6
     */
    logLevel?: number;
}
export interface PanelUserConfig {
    username: string;
    password: string;
}
export interface PanelOptions {
    /**
     * @description
     * Specifies if the panel should be active.
     * @default false
     */
    active?: boolean;
    /**
     * @description
     * Specifies the panel user/s.
     * @example
     * user: {username: 'admin', password: 'admin'}
     * user: [{username: 'admin', password: 'admin'},{username: 'user',password: '12345'}]
     */
    user?: PanelUserConfig | PanelUserConfig[];
}
export interface InternalServerConfig extends ServerConfig {
    id: string;
    port: number;
    hostname: string;
    path: string;
    origins: string | null | string[];
    tags: ([string, string][]) | string;
    debug: boolean;
    launchDebug: boolean;
    showConfigWarnings: boolean;
    checkConfigs: boolean;
    timeZone: string;
    tls: TLSOptions | null;
    sendErrorDescription: boolean;
    defaultClientApiLevel: number;
    panel: PanelOptions;
    auth: AuthOptions;
    compression: CompressionOptions;
    validationCheckLimit: number;
    socketDataboxLimit: number;
    socketChannelLimit: number;
    variables: any;
    log: DeepRequired<LogOptions>;
    logComponentApi: boolean;
    responseTimeout: number;
    pingInterval: number;
    maxPayloadSize: number;
    maxBackpressure: number;
    binaryContentPacketTimeout: number;
    streamsPerPackageLimit: number;
    chunksCanContainStreams: boolean;
    join: string | null;
    clusterShareTokenAuth: boolean;
    brokerClusterClientMaxPoolSize: number;
}
