import { Server } from "ziron-worker";
import Socket from "../../../../api/Socket";
import CodeError from "../../../error/codeError";
import { RawSocket } from '../../../definitions/rawSocket';
export declare type LaunchedFunction = () => Promise<void> | void;
export declare type ExitFunction = () => Promise<void> | void;
export declare type ErrorFunction = (error: Error) => Promise<void> | void;
export declare type CodeErrorFunction = (codeError: CodeError) => Promise<void> | void;
export declare type ServerMessageFunction = (data: any) => Promise<void> | void;
export declare type SocketConnectionFunction = (socket: Socket) => Promise<void> | void;
export declare type SocketDisconnectionFunction = (socket: Socket, code: any, data: any) => Promise<void> | void;
export declare type SocketAuthenticationFunction = (socket: Socket) => Promise<void> | void;
export declare type SocketDeauthenticationFunction = (socket: Socket) => Promise<void> | void;
export declare type CoreServerErrorFunction = (error: Error, server: Server<RawSocket>) => Promise<void> | void;
export declare type CoreSocketErrorFunction = (error: object, socket: Socket) => Promise<void> | void;
export declare type SocketBadAuthTokenFunction = (socket: Socket, error: Error, signedAuthToken: string) => Promise<void> | void;
export declare type Event<T> = T | T[];
export interface Events {
    /**
     * This event gets triggered when the server has launched successfully.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example () => {}
     */
    launched?: Event<LaunchedFunction>;
    /**
     * This event gets triggered when the server is about to exit.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example () => {}
     */
    exit?: Event<ExitFunction>;
    /**
     * This event gets triggered when an unknown error is thrown on the server while:
     * processing a request, processing a middleware,
     * run a background task or processing another event.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (error) => {}
     */
    error?: Event<ErrorFunction>;
    /**
     * This event gets triggered when a CodeError is created.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (codeError) => {}
     */
    codeError?: Event<CodeErrorFunction>;
    /**
     * This event gets triggered when the server receives a server message
     * that was sent from the bag.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (data) => {}
     */
    serverMessage?: Event<ServerMessageFunction>;
    /**
     * This event gets triggered when a new socket is connected to the server.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socketConnection?: Event<SocketConnectionFunction>;
    /**
     * This event gets triggered when a socket gets disconnected.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket,code,data) => {}
     */
    socketDisconnection?: Event<SocketDisconnectionFunction>;
    /**
     * This event gets triggered when a socket gets authenticated or the auth token has changed.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socketAuthentication?: Event<SocketAuthenticationFunction>;
    /**
     * This event gets triggered when a socket gets deauthenticated.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socketDeauthentication?: Event<SocketDeauthenticationFunction>;
    /**
     * This event gets triggered when an error occurs on the core server.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (error,server) => {}
     */
    coreServerError?: Event<CoreServerErrorFunction>;
    /**
     * This event gets triggered when an error occurs on a core socket.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (error,socket) => {}
     */
    coreSocketError?: Event<CoreSocketErrorFunction>;
    /**
     * This event gets triggered when a client tries to authenticate itself with an invalid (or expired) token.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket,error,signedAuthToken) => {}
     */
    socketBadAuthToken?: Event<SocketBadAuthTokenFunction>;
}
export interface PreparedEvents extends Events {
    launched: LaunchedFunction;
    exit: ExitFunction;
    error: ErrorFunction;
    codeError: CodeErrorFunction;
    serverMessage: ServerMessageFunction;
    socketConnection: SocketConnectionFunction;
    socketDisconnection: SocketDisconnectionFunction;
    socketAuthentication: SocketAuthenticationFunction;
    socketDeauthentication: SocketDeauthenticationFunction;
    coreServerError: CoreServerErrorFunction;
    coreSocketError: CoreSocketErrorFunction;
    socketBadAuthToken: SocketBadAuthTokenFunction;
}
