import ZationToken from '../../../internalApi/zationToken';
import Socket from '../../../../api/Socket';
import { FlowControlledMiddlewareInvoker, StandardMiddlewareInvoker } from '../../../middlewares/middlewaresPreparer';
import { HttpRequest, HttpResponse, Server } from "ziron-worker";
import { RawSocket } from "../../../definitions/rawSocket";
export declare type MiddlewareAuthenticationFunction = (token: ZationToken) => Promise<boolean | object | any> | boolean | object | any;
export declare type MiddlewareSocketFunction = (socket: Socket) => Promise<boolean | object | any> | boolean | object | any;
export declare type MiddlewarePanelAuthFunction = (username: string, password: string) => Promise<boolean | void> | boolean | void;
export declare type MiddlewareInitFunction = () => Promise<void> | void;
export declare type MiddlewareBeforeComponentsInitFunction = () => Promise<void> | void;
export declare type MiddlewareAfterComponentsInitFunction = () => Promise<void> | void;
export declare type MiddlewareCoreServerInitFunction = (server: Server<RawSocket>) => Promise<void> | void;
export declare type MiddlewareSocketInitFunction = (socket: Socket) => Promise<void> | void;
export declare type MiddlewareCleanupFunction = () => Promise<void> | void;
export declare type MiddlewareHttpRequestFunction = (req: HttpRequest, res: HttpResponse) => Promise<void> | void;
export declare type Middleware<T> = T[] | T;
export interface Middlewares {
    /**
     * Flow control middleware where you can block wrong jwt tokens.
     * You can provide one function or multiple middleware functions.
     * When providing multiple functions, they will be invoked in the defined sequence.
     * If one function returns or throws some value, the chain will be broken,
     * and the value is the result.
     * That means if you return nothing, the next function will be called.
     * If no more function is remaining, the action will be allowed.
     * If one function returns true, the chain is broken,
     * and the token is allowed without invoking the remaining functions.
     * To block the token, you can return a Block instance or false or throwing a Block instance.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (zationToken) => {}
     */
    authenticate?: Middleware<MiddlewareAuthenticationFunction>;
    /**
     * Flow control middleware where you can block sockets.
     * You can provide one function or multiple middleware functions.
     * When providing multiple functions, they will be invoked in the defined sequence.
     * If one function returns or throws some value, the chain will be broken,
     * and the value is the result.
     * That means if you return nothing, the next function will be called.
     * If no more function is remaining, the action will be allowed.
     * If one function returns true, the chain is broken,
     * and the socket is allowed without invoking the remaining functions.
     * To block the socket, you can return a Block instance or false or throwing a Block instance.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socket?: Middleware<MiddlewareSocketFunction>;
    /**
     * Flow control middleware where you can allow or block
     * authentication requests with credentials to the panel.
     * This is useful if you want to change user accounts dynamically or
     * connect them to users of a database.
     * The middleware will only be used after Zation was not able
     * to authenticate the user with the users defined in the server config.
     * You can provide one function or multiple middleware functions.
     * When providing multiple functions, they will be invoked in the defined sequence.
     * If one function returns or throws some value, the chain will be broken,
     * and the value is the result.
     * That means if you return nothing, the next function will be called.
     * If no more function is remaining, the authentication request is blocked.
     * If one function returns true, the chain is broken,
     * and the authentication request is successful without
     * invoking the remaining functions.
     * To deny the authentication request, you can return a Block instance or
     * false or throwing a Block instance.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (username, password) => {}
     */
    panelAuth?: Middleware<MiddlewarePanelAuthFunction>;
    /**
     * This standard (unchained) middleware lets you initiate
     * additional things on the startup of the server.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example async () => {}
     */
    init?: Middleware<MiddlewareInitFunction>;
    /**
     * This standard (unchained) middleware will be processed before all
     * components are initialized.
     * It is helpful to prepare things for the initialization of the components.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example async () => {}
     */
    beforeComponentsInit?: Middleware<MiddlewareBeforeComponentsInitFunction>;
    /**
     * This standard (unchained) middleware will be processed after all
     * components are initialized.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example async () => {}
     */
    afterComponentsInit?: Middleware<MiddlewareAfterComponentsInitFunction>;
    /**
     * This standard (unchained) middleware lets you initialize extra
     * things on the core server during the server startup.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (server) => {}
     */
    coreServerInit?: Middleware<MiddlewareCoreServerInitFunction>;
    /**
     * This standard (unchained) middleware lets you initialize extra
     * things in the creation process of a socket.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (socket) => {}
     */
    socketInit?: Middleware<MiddlewareSocketInitFunction>;
    /**
     * This standard (unchained) middleware lets you clean up resources before the server exits.
     * Notice that long-running processes could be terminated.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example () => {}
     */
    cleanup?: Middleware<MiddlewareCleanupFunction>;
    /**
     * This standard (chained) middleware lets you handle custom HTTP
     * requests that could not be caught and processed internally.
     * The chain will break when the HTTP response has been ended,
     * aborted or reserved.
     * If the HTTP request has not been answered after the middleware,
     * a 426 (Upgrade Required) response will be sent.
     * Notice that the given request and response object
     * is not from the standard Node.js HTTP server and is more difficult to use.
     * Use it only for advanced use cases.
     * The Bag instance can be securely accessed with the variable 'bag'.
     * @example (req,res) => {}
     */
    httpRequest?: Middleware<MiddlewareHttpRequestFunction>;
}
export interface PreparedMiddlewares {
    authenticate: FlowControlledMiddlewareInvoker<MiddlewareAuthenticationFunction>;
    socket: FlowControlledMiddlewareInvoker<MiddlewareSocketFunction>;
    panelAuth: FlowControlledMiddlewareInvoker<MiddlewarePanelAuthFunction>;
    init: StandardMiddlewareInvoker<MiddlewareInitFunction>;
    beforeComponentsInit: StandardMiddlewareInvoker<MiddlewareBeforeComponentsInitFunction>;
    afterComponentsInit: StandardMiddlewareInvoker<MiddlewareAfterComponentsInitFunction>;
    coreServerInit: StandardMiddlewareInvoker<MiddlewareCoreServerInitFunction>;
    socketInit: StandardMiddlewareInvoker<MiddlewareSocketInitFunction>;
    cleanup: StandardMiddlewareInvoker<MiddlewareCleanupFunction>;
    httpRequest: StandardMiddlewareInvoker<MiddlewareHttpRequestFunction>;
}
